local T = ZZT_ZombieTag

T.isIt = function(player)
	return player.zzt_isIt == true
end

T.isInGamemode = function()
	return gametype == GT_ZOMBIETAG
end

T.isPlayerActive = function(player)
	return player.mo and not player.spectator
end

T.getZTAbilityData = function(player)
	if not player or not player.valid then return nil end

	local function getModified(role, abilityId)
		local ability = T.ZT_Abilities[role] and T.ZT_Abilities[role][abilityId]
		if not ability then return nil end

		if T.zzt_mod_blitz == true then
			local modified = {}
			for k, v in pairs(ability) do
				modified[k] = v
			end
			if modified.ringCost then
				modified.ringCost = modified.ringCost / 2
			end
			if modified.cooldown then
				modified.cooldown = modified.cooldown / 2
			end
			return modified
		end

		return ability
	end

	return {
		runner = getModified("runner", player.zrunnerability),
		tagger = getModified("tagger", player.ztaggerability)
	}
end

T.beforeStart = function(player)
		local hidetime = CV_FindVar("hidetime").value or 10
		local allowedTime = hidetime * TICRATE
		if leveltime > allowedTime then return end
end

T.Reset = function(player)
	//Initialization
	player.zzt_cooldown = 0
	if player.zrunnerability == nil
	player.zrunnerability = 1
	end
	if player.ztaggerability == nil
	player.ztaggerability = 1
	end
	if player.zzt_storerunability == nil
	player.zzt_storerunability = 1
	end
	if player.zzt_storezomability == nil
	player.zzt_storezomability = 1
	end
	if player.zzt_grounded == nil
	player.zzt_grounded = 0
	end
	if player.zzt_storedposition == nil
	player.zzt_storedposition = 0
	end
	if player.zzt_nocontrol == nil
	player.zzt_nocontrol = 0
	end
	if player.zzt_bombDead == nil
	player.zzt_bombDead = false
	end
	player.zzt_vote = nil
	player.zzt_hudMsgTime = 0
	player.Juggset = false
end

T.Init_Buttons = function(player)
	if player.ert_buttonhistory == nil then
		player.ert_buttonhistory = player.cmd.buttons
	end	
end



T.ShieldRemove = function(player)
	player.powers[pw_shield] = 0
end

local function padRight(str, len)
	str = tostring(str)
	local strLen = #str
	local padLen = (strLen < len) and (len - strLen) or 0
	return str .. string.rep(" ", padLen)
end

local function intDiv(a, b)
    return (a - (a % b)) / b
end

local function roundToNearest10(n)
    local remainder = n % 10
    if remainder >= 5 then
        return n + (10 - remainder)
    else
        return n - remainder
    end
end

T.formatCooldown = function(cooldownTicks, TICRATE)
    if not cooldownTicks or cooldownTicks == 0 then
        return "0 sec"
    end

    -- multiply by 100 to keep two decimals as integer
    local raw = intDiv(cooldownTicks * 100, TICRATE)

    -- round raw to nearest 10 (nearest tenth of a second)
    local rounded = roundToNearest10(raw)

    -- now split to seconds and tenths
    local seconds = intDiv(rounded, 100)
    local tenths = intDiv(rounded % 100, 10)

    if tenths == 0 then
        return tostring(seconds) .. " sec"
    else
        return tostring(seconds) .. "." .. tostring(tenths) .. " sec"
    end
end

T.printAbilities = function(player, category)
	local abilities = T.ZT_Abilities[category]
	if not abilities then
		CONS_Printf(player, "No abilities found for category: " .. tostring(category))
		return
	end

	-- Color codes
	local blue   = string.char(130)
	local red    = string.char(133)
	local gray   = string.char(128)
	local green  = string.char(138)
	local yellow = string.char(131)

	local headerColor = (category == "runner") and blue or red
	CONS_Printf(player, headerColor.."Available "..category:upper().." Abilities:\n")

	local labelWidth = 18
	local nameWidth = 22

	for id, info in ipairs(abilities) do
		-- Debug print raw cooldown value

		local abilityLabel = string.format("%-" .. labelWidth .. "s", category.."ability "..id)
		local name = string.format("%-" .. nameWidth .. "s", info.name or "")
		local ringCostStr = string.format("%2s", tostring(info.ringCost))
		local cooldownStr = T.formatCooldown(info.cooldown, TICRATE)

		local line =
			green .. abilityLabel ..
			yellow .. name ..
			blue .. ringCostStr .. " Rings " ..
			gray .. "| Cooldown: " .. cooldownStr

		CONS_Printf(player, line)

		if info.desc and #info.desc > 0 then
			CONS_Printf(player, gray.."    " .. info.desc)
		end

		CONS_Printf(player, "")
	end
end



T.AbilityCheck = function(player)
	if not T.isInGamemode() then return end
	local allowedTime = 15 * TICRATE

	if leveltime < allowedTime then
		player.ztaggerability = player.zzt_storezomability
		player.zrunnerability = player.zzt_storerunability
		player.zzt_isIt = false
		player.powers[pw_invulnerability] = 1
	end
end

addHook("MapThingSpawn", function(mobj)
	if not T.isInGamemode() then return end
	mobj.type = MT_RING_BOX
end,MT_RING_REDBOX)
addHook("MapThingSpawn", function(mobj)
	if not T.isInGamemode() then return end
	mobj.type = MT_RING_BOX
end,MT_RING_BLUEBOX)