local T = ZZT_ZombieTag
local sound_freeze = sfx_cdfm51
local sound_unfreeze = sfx_wbreak

T.makeFrozen = function(pl)
	if pl.zzt_frozentimer == nil then pl.zzt_frozentimer = 60*TICRATE end
	pl.pflags = pl.pflags | PF_INVIS
	pl.powers[pw_shield] = 0
	pl.powers[pw_flashing] = 100
	pl.powers[pw_nocontrol] = 32767
	S_FadeMusic(50, MUSICRATE/2, pl)
end

T.makeSuperFrozen = function(pl)
	T.makeFrozen(pl)
	pl.superfrozen = true
end

T.makeUnfrozen = function(pl)
	pl.pflags = pl.pflags & ~PF_INVIS
	pl.powers[pw_nocontrol] = 0
	pl.powers[pw_flashing] = 0
	pl.powers[pw_underwater] = 23 * TICRATE
	
	pl.mo.state = S_PLAY_STND
	pl.mo.tics = 1
	pl.superfrozen = false
	
	S_StopFadingMusic(pl)
	S_SetInternalMusicVolume(100, pl)
end

T.isFrozen = function(pl)
	return (pl.pflags & PF_INVIS) ~= 0
end

T.isSuperFrozen = function(pl)
	return pl.superfrozen
end

T.FrozenDrain = function(pl)
	if T.isFrozen(pl) then
		pl.zzt_frozentimer = $-1
		local hp = pl.zzt_frozentimer/TICRATE
		T.ShowHUDMessage(pl, "\x88Your health is " .. hp .. "!", TICRATE)
			if hp <= 0 then
				pl.zzt_isIt = true
				T.makeUnfrozen(pl)
			end
	end
end

T.frozenspawner = function(pl)
		if T.isFrozen(pl) then
			pl.mo.anim_duration = UINT16_MAX
			if pl.powers[pw_flashing] == 0 then pl.powers[pw_flashing] = 100 end

				local mobj = P_SpawnMobj(pl.mo.x, pl.mo.y, pl.mo.z, MT_TAG)
			mobj.sprite = SPR_CEMG
			if T.isSuperFrozen(pl) then
				mobj.frame = 2 | FF_TRANS50 | FF_FULLBRIGHT
			else
				mobj.frame = 3 | FF_TRANS50 | FF_FULLBRIGHT
			end
			mobj.tics = 1
			mobj.scale = FRACUNIT*3
      
			pl.powers[pw_nocontrol] = 32767
			pl.powers[pw_underwater] = UINT16_MAX
      
			pl.mo.state = S_PLAY_DEAD
		end
end


T.doFreeze = function(taggerPl, targetPl)
	if not (taggerPl and taggerPl.mo and targetPl and targetPl.mo) then return end
	if targetPl.powers[pw_invulnerability] ~= 0 then return end

	if taggerPl.powers[pw_invulnerability] ~= 0 then
		T.makeSuperFrozen(targetPl)
	else
		T.makeFrozen(targetPl)
	end

	if targetPl.mo and targetPl.mo.valid and targetPl.climbing ~= 1 then
		P_DoPlayerPain(targetPl, taggerPl.mo)
	end

	P_PlayerEmeraldBurst(targetPl)
	S_StartSound(targetPl.mo, sound_freeze, targetPl)
	print("\x85" .. taggerPl.name .. "\x80 froze \x84" .. targetPl.name .. "\x80!")

	return true
end


T.doUnfreeze = function(mo1,mo2)
	P_AddPlayerScore(mo1.player, 25)
	
	T.makeUnfrozen(mo2.player)
	
	mo2.player.powers[pw_invulnerability] = 1*TICRATE
		 	
	S_StartSound(mo2, sound_unfreeze, pl)
	print("\x84"+mo1.player.name+"\x80 unfroze \x84"+mo2.player.name+"\x80!")
	return true
end

T.HandleFreezeInteraction = function(taggerPl, targetPl)
	if not (taggerPl and targetPl) then return end
	if not taggerPl.mo or not targetPl.mo then return end
	if not T.isInGamemode() or T.zzt_gamemode ~= "freeze" then return end

	-- Tagger is It, target is not frozen
	if T.isIt(taggerPl) and not T.isIt(targetPl) then
		if not T.isFrozen(targetPl) then
			T.doFreeze(taggerPl, targetPl)
		end
		if targetPl.powers[pw_invulnerability] ~= 0 then
			P_DoPlayerPain(taggerPl, targetPl.mo)
		end
		return true
	end
	-- Anti-camp
	if T.isIt(targetPl) and T.isFrozen(taggerPl)
		and taggerPl.mo.momx == 0 and taggerPl.mo.momy == 0
		and targetPl.rings >= 5 and leveltime % 2 == 0
	then
		targetPl.rings = $ - 5
		taggerPl.rings = $ + 5
	end
end

addHook("MobjCollide", function(mo2,mo1)
	if not T.isInGamemode() then return end
	if not T.zzt_gamemode == "frozen" then return end
	if not mo1.player or not mo2.player then return end
	if not (mo1.z+mo1.height > mo2.z and mo2.z+mo2.height > mo1.z) then return end
       
	local pl1,pl2 = mo1.player,mo2.player
	if not T.isIt(pl1) and not T.isIt(pl2)
       and T.isFrozen(pl2) and not T.isFrozen(pl1) then
		local superfrz = T.isSuperFrozen(pl2)
		local unfroze = T.doUnfreeze(mo1,mo2)
		pl1.score = $ + 50
		if unfroze and superfrz then
			P_DoPlayerPain(mo1.player,mo2)
		end
	end
       
	--Anti-camp.
	if T.isIt(pl2) and T.isFrozen(pl1)
       and pl1.mo.momx == 0
       and pl1.mo.momy == 0
       and pl2.rings >= 5
       and leveltime % 2 == 0 then 
       
       pl2.rings = $-5
       pl1.rings = $+5
	end
end, MT_PLAYER)