local T = ZZT_ZombieTag

//Variables
local color_it = 37
local color_runner = 150
local color_frozen = 130
local color_unassigned = 0
local statusStr = "WAITING!"
local statusColor = V_GRAYMAP
//Change these according to gamemode

-- [[Radar+Hud]]
T.RadarHud = function(v, player, cam)
	if not T.isInGamemode() then return end
	if player.playerstate ~= PST_LIVE then return end

	local xoffset = 8
	local yoffset = 128
	local textY = 112
	local flags = V_PERPLAYER
	local radarscale=(45*FRACUNIT)
	local radarwarn=0
	local gm = T.zzt_gamemode

	-- Determine status string
	if T.beforeStart() then
			statusStr = "WAITING!"
			statusColor = V_GRAYMAP
	elseif gm == "zombie" then
			statusStr = "RUNNER!"
			statusColor = V_BLUEMAP
			if T.isIt(player) then
				statusStr = "ZOMBIE!"
				statusColor = V_GREENMAP
			end
			color_it = 100
			color_runner = 150
	elseif gm == "freeze" then
			statusStr = "RUNNER!"
			statusColor = V_BLUEMAP
			if T.isIt(player) then
				statusStr = "FREEZER!"
				statusColor = V_AQUAMAP
			end
			if T.isFrozen(player) then
				statusStr = "FROZEN!"
				statusColor = V_SKYMAP
			end
			color_it = 122
			color_runner = 150
			
	elseif gm == "crown" then
			statusStr = "CROWNED!"
			statusColor = V_YELLOWMAP
			if T.isIt(player) then
				statusStr = "PEASANT!"
				statusColor = V_BROWNMAP
			end
			color_it = 230
			color_runner = 73
	elseif gm == "tag" then
			statusStr = "RUNNER!"
			statusColor = V_BLUEMAP
			if T.isIt(player) then
				statusStr = "TAGGER!"
				statusColor = V_REDMAP
			end
			color_it = 37
			if T.zzt_mod_disguised == true then color_it = 150 end
			color_runner = 150
	elseif gm == "bomb" then
		color_it = 57
		color_runner = 150
		color_frozen = 28

		if player.zzt_bombDead then
			statusStr = "DEAD"
			statusColor = V_GRAYMAP
		else
			statusStr = "RUNNER!"
			statusColor = V_BLUEMAP
			if T.isIt(player) then
				statusStr = "BOMB!"
				statusColor = V_ORANGEMAP
			end
		end
	elseif gm == "juggernaut" then
		color_it = 37
		color_runner = 187
		statusStr = "JUGGERNAUT!"
		statusColor = V_PURPLEMAP
		if T.isIt(player) then
		statusStr = "HUNTER!"
		statusColor = V_REDMAP
		end
	end


	v.drawString(xoffset, textY, statusStr, statusColor | flags)

	-- Handle ring color logic
	local ringColor = V_WHITEMAP
	local timeLeft = nil

	if player.rings >= 50 and not player.powers[pw_super] then
		ringColor = V_YELLOWMAP
	end

	if player.isxmomentum and player.rings >= 150 and not player.powers[pw_super] then
		ringColor = V_SKYMAP
	end

	if player.powers[pw_super] then
		timeLeft = (supertimer - player.powers[pw_super]/TICRATE)
		if T.isIt(player) then
			timeLeft = (2 * supertimer - player.powers[pw_super]/TICRATE)
		end
		if timeLeft >= 10 then
			ringColor = skincolors[player.mo.color].chatcolor
		elseif timeLeft < 10 then
			ringColor = V_REDMAP
		end
	end

	-- Draw ring count
	v.drawString(xoffset, textY + 8, "RINGS: " .. "\x82" .. player.rings, flags)

	-- Draw super/hyper time
	if player.powers[pw_super] then
		local superLabel = player.isxmomentum and player.hyper.capable and "HYPER: " or "SUPER: "
		v.drawString(xoffset, textY - 16, superLabel .. tostring(timeLeft))
	end

	-- T.radar drawing
	if T.radar and player.radar ~= 0 then
		local radarX = xoffset
		local radarY = yoffset
		local angle = (-cam.angle) - ANGLE_90

		for other in players.iterate do
			if other.playerstate ~= PST_LIVE or not T.isPlayerActive(other) then continue end

			local dx = other.mo.x - player.mo.x
			local dy = other.mo.y - player.mo.y

			local x = -(FixedMul(dx, cos(angle)) - FixedMul(dy, sin(angle))) / radarscale
			local y =  (FixedMul(dx, sin(angle)) + FixedMul(dy, cos(angle))) / radarscale

			-- Visibility range
			if (x > 31) or (x < -31) or (y > 31) or (y < -31) then
				if not T.isIt(player) and not T.isIt(other) then continue end
				if T.isIt(player) and T.isIt(other) then continue end
			end

			-- Clamp
			x = max(-31, min(31, x))
			y = max(-31, min(31, y))

			local blink = (leveltime % 4 == 0)
			local color = color_unassigned

			if T.isFrozen(other) or other.zzt_bombDead then
				color = color_frozen -- <- define this if you haven't
			elseif not T.isIt(other) then
				color = color_runner
			elseif T.isIt(other) then
				color = color_it
			end

			v.drawFill(radarX + 30 + x, radarY + 30 + y, 4, 4, color | flags)
		end

		-- T.radar frame
		local frameColor = radarwarn and 7 or 3
		v.drawFill(radarX, radarY,      64, 1, frameColor | flags)
		v.drawFill(radarX, radarY + 63, 64, 1, frameColor | flags)
		v.drawFill(radarX, radarY,      1, 64, frameColor | flags)
		v.drawFill(radarX + 63, radarY, 1, 64, frameColor | flags)
	end

	T.TopText(v, player)
	if guide == 1 then drawTooltips(v, player)
	elseif guide == 2 then drawWelcome(v, player)
	end
end


--[[Gamemode Constants]]


local function countPlayers()
	local count,it=0,0,0
	for pl in players.iterate
		if pl.spectator then continue end
		
		if isIt(pl) then it=$+1 end
		count=count+1
	end
	return count,it
end


T.disableHud = function(player)
	if T.isInGamemode() then
		hud.disable("weaponrings")
		hud.disable("score")
		hud.disable("rings")
		hud.disable("time")
		hud.disable("lives")
	end
end

--[[HUD]]
T.TopText = function(v, player)
	if not T.isInGamemode() then return end

	local flags = V_PERPLAYER
	local color
	local gametime = timelimit * 60 * TICRATE

	-- Get dynamic hide time
	local hideTime = 15
	local hideTicks = hideTime * TICRATE

	-- Activate T.radar halfway through match + hideTime grace
	if T.radarhalf == 1 then
		if leveltime >= (gametime / 2) + hideTime then
			radar = 1
		else
			radar = 0
		end
	end

	-- Pre-round countdown
	if leveltime <= hideTicks then
		local timeLeft = hideTime - (leveltime / TICRATE)

			v.drawString(160, 0, "HAVE",     (color or V_YELLOWMAP) | flags, "center")
			v.drawString(160, 8, "FUN",   color or 0 | flags, "center")
			v.drawString(160, 16, tostring(timeLeft), V_REDMAP | flags, "center")
	else
		local timeRemaining = ((gametime - leveltime) / TICRATE)
		v.drawString(160, 0, "Time:", (color or V_YELLOWMAP) | flags, "center")
		v.drawString(160, 8, tostring(timeRemaining), color or 0 | flags, "center")
	end



    local role = player.zzt_roleSelect or "runner"
    local list = T.ZT_Abilities[role]
	local abilities = T.ZT_Abilities[category]
    if list then
	local allowedTime = 15 * TICRATE
	if leveltime > allowedTime then return end
        local index = player.zzt_abilityIndex or 1
        local ab = list[index]
        if ab then
            local y = 50

            local prev = list[(index - 2 + #list) % #list + 1]
            local next = list[(index % #list) + 1]

            if prev then
                v.drawString(80, y + 5, prev.name or "", V_60TRANS | flags, "center")
            end
            if next then
                v.drawString(240, y + 5, next.name or "", V_60TRANS | flags, "center")
            end

            v.drawString(160, y, ab.name or "???", V_YELLOWMAP | flags, "center")
            v.drawString(160, y + 10, "Rings: " .. tostring(ab.ringCost), V_ORANGEMAP | flags, "center")

            local function max(a,b) return (a > b) and a or b end
            local cd = T.formatCooldown(ab.cooldown, TICRATE)
			
            v.drawString(160, y + 20, "Cooldown: " .. string.format(cd) .. "s", V_BLUEMAP | flags, "center")

            v.drawString(160, y + 30, "[" .. role:upper() .. " ABILITIES]", V_50TRANS | flags, "center")
        end
    end
end

local function drawTooltips(g,pl)
	local i=0
	
	for k,v in pairs(tooltips["general"]) do
		drawTooltip(g,pl,i,v)
		i=i+1
	end

	local state = "idle"
	if leveltime > 0 then
		if T.isIt(pl) then
			state = "it"
		end
		if not T.isIt(pl) then
			state = "runner"
		end
		
	end
	for k,v in pairs(tooltips[state]) do
		drawTooltip(g,pl,i,v)
		i=i+1
	end
end

T.HUD = function(v, player, cam)
	if player.playerstate ~= PST_LIVE then return end
	if not T.isInGamemode() then return end

	local TF_GRAY   = 1
	local TF_YELLOW = 2
	local TF_RED    = 3

	local xoffset = -16
	local yoffset = 104
	local flags = V_PERPLAYER

	local function drawAbilityEntry(role, abilityId, cooldown, rings, y)
		local entry = T.ZT_Abilities[role] and T.ZT_Abilities[role][abilityId]
		if not entry then return end

		local name = entry.name
		local cost = entry.ringCost or 0
		local cd = entry.cooldown or 0

		-- Blitz mod adjustment
		if T.zzt_mod_blitz then
			cost = cost / 2
			cd = cd / 2
		end

		local text = name
		local textflags = 0
		local colorPrefix = "\x80" -- default white

		if cooldown and cooldown > 0 then
			textflags = TF_GRAY
			text = "Cooldown " ..
				G_TicsToSeconds(cooldown) .. "." ..
				G_TicsToCentiseconds(cooldown)
		elseif cost > rings then
			textflags = TF_GRAY
		end

		if textflags == TF_GRAY then
			colorPrefix = "\x86"
		elseif textflags == TF_YELLOW then
			colorPrefix = "\x82"
		elseif textflags == TF_RED then
			colorPrefix = "\x85"
		end

		if (not cooldown or cooldown == 0) and cost then
			text = "\x82" .. cost .. "\x80 " .. colorPrefix .. text
		else
			text = colorPrefix .. text
		end

		local patch = v.cachePatch("THRWRING")
		v.draw(xoffset, y, patch, flags)
		v.drawString(xoffset + 12, y, text, flags)
	end

	-- Draw both roles stacked (10 pixels apart)
	drawAbilityEntry("runner", player.zrunnerability, player.zzt_runnerCooldown or 0, player.rings, yoffset)
	drawAbilityEntry("tagger", player.ztaggerability, player.zzt_taggerCooldown or 0, player.rings, yoffset - 10)
end




addHook("PlayerThink", function(pl)
if T.radar then pl.radar = 1 end
if T.tagradaronly and T.isIt(pl) then pl.radar = 0 end
end)


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


--Restrict Super time
//Code borrowed from Zoraxua's Super in Ringslinger
addHook("PlayerThink", function(pl)
	if gametype == GT_ZOMBIETAG
	//Local variables to help the game
		if pl.iwantemmymusic == nil then pl.iwantemmymusic = 0 end
		if pl.myemmymusicis == nil then pl.myemmymusicis = "_INV" end
	//Poggers

		//Now what's the requirements?
		if pl.powers[pw_super] //Don't wanna remove super mid-use
			if T.isIt(pl)
				if pl.powers[pw_super] == 2*T.supertimer*TICRATE
					pl.powers[pw_super] = 0 //So lose super if you've had it for too long, grr
					pl.rings = 0
					pl.powers[pw_emeralds] = 0
					//Following ripped from SR lmao
					S_StartSound(nil, sfx_s3k9c) //Trans SFX, global so everyone knows!
					pl.mo.color = pl.skincolor //Fix supercolor
					pl.mo.state = pl.mo.state //Go back to normal if you have super sprites
					P_SpawnShieldOrb(pl) //Get the sheild back if you had one
					P_RestoreMusic(pl) //Fix music if character uses custom for soup
				end
			end
			if not T.isIt(pl)
				if pl.powers[pw_super] == T.supertimer*TICRATE
					pl.powers[pw_super] = 0 //So lose super if you've had it for too long, grr
					pl.rings = 0
					pl.powers[pw_emeralds] = 0
					//Following ripped from SR lmao
					S_StartSound(nil, sfx_s3k9c) //Trans SFX, global so everyone knows!
					pl.mo.color = pl.skincolor //Fix supercolor
					pl.mo.state = pl.mo.state //Go back to normal if you have super sprites
					P_SpawnShieldOrb(pl) //Get the sheild back if you had one
					P_RestoreMusic(pl) //Fix music if character uses custom for soup
				end
			end
		end
	end
end)
