local T = ZZT_ZombieTag

T.DoTag = function(taggerPl, targetPl)
	if not (taggerPl and targetPl) then return end
	if targetPl.powers[pw_invulnerability] > 0 then return end

	local gm = T.zzt_gamemode or "zombie"

	if gm == "tag" then
		-- Normal tag: swap roles, give invulnerability to previous It
		targetPl.zzt_isIt = true
		taggerPl.zzt_isIt = false
		taggerPl.powers[pw_invulnerability] = TICRATE -- 1 second invuln
		taggerPl.score = $ + 50
		taggerPl.zzt_cooldown = 0
		targetPl.zzt_cooldown = 0
		T.ShowHUDMessage(taggerPl, "\x84Tagged "..targetPl.name.."!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "\x85You were tagged by "..taggerPl.name.."!", 2*TICRATE)
	elseif gm == "zombie" then
		-- Zombie tag: infect target (turn them It), original It stays It
		targetPl.zzt_isIt = true
		taggerPl.score = $ + 100
		targetPl.zzt_cooldown = 0
		T.ShowHUDMessage(taggerPl, "\x83Infected "..targetPl.name.."!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "\x83You were infected by "..taggerPl.name.."!", 2*TICRATE)
	elseif gm == "freeze" then
		-- Freeze tag: freeze target, original stays It
		T.HandleFreezeInteraction(taggerPl, targetPl)
		taggerPl.score = $ + 50
		targetPl.zzt_cooldown = $/2
		T.ShowHUDMessage(taggerPl, "Froze "..targetPl.name.."!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "You were frozen by "..taggerPl.name.."!", 2*TICRATE)
	elseif gm == "bomb" and targetPl.zzt_bombDead == false then
		targetPl.zzt_isIt = true
		taggerPl.zzt_isIt = false
		taggerPl.powers[pw_invulnerability] = TICRATE
		taggerPl.zzt_cooldown = 0
		targetPl.zzt_cooldown = 0

		-- Add 5 seconds to the bomb timer on pass
		T.bombTimer = $ + (5 * TICRATE)

		T.ShowHUDMessage(taggerPl, "You gave "..targetPl.name.." the bomb!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "\x85"..taggerPl.name.." gave you the bomb!", 2*TICRATE)
	elseif gm == "crown" then
		-- Trade tag: give/take form
		targetPl.zzt_isIt = true
		taggerPl.zzt_isIt = false
		taggerPl.powers[pw_invulnerability] = TICRATE
		taggerPl.zzt_cooldown = 0
		targetPl.zzt_cooldown = 0
		T.ShowHUDMessage(taggerPl, "\x82Stole "..targetPl.name.."'s crown!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "\x85Your crown was stolen by "..taggerPl.name.."!", 2*TICRATE)
	elseif gm == "juggernaut" then
		targetPl.score = $ - 10
		T.JuggernautHP = targetPl.score
		targetPl.powers[pw_invulnerability] = TICRATE
		T.ShowHUDMessage(taggerPl, "\x82Damaged"..targetPl.name.."'s health!", 2*TICRATE)
		T.ShowHUDMessage(targetPl, "\x85Your health is "..targetPl.score.."!", 2*TICRATE)
	end

	P_DoPlayerPain(targetPl)


	if T.zzt_gamemode == "zombie" or T.zzt_gamemode == "freeze" or T.zzt_gamemode == "juggernaut" then
		T.CheckRoundEnd()
	end
end

T.CustomMusic = 0

T.IsLastSurvivor = function(player)
	if T.zzt_gamemode ~= "zombie" then return false end
	if not player.mo or not player.valid or player.spectator then return false end
	if T.isIt(player) then return false end

	-- Count other alive survivors
	for other in players.iterate do
		if other ~= player 
		and other.mo and other.valid 
		and not other.spectator 
		and not T.isIt(other) then
			-- Found another survivor, so player is NOT last
			return false
		end
	end
	--Insert buff here.
	if leveltime % 35 == 0
		player.rings = $+1
	end

	local musicName = T.PlayerCustom[player.name:lower()] -- lowercase to match keys
	if musicName and T.CustomMusic == 0 then
		-- Play for this player
		P_PlayJingleMusic(player, musicName, nil, true, JT_OTHER)
		
		-- Play for everyone else too
		for other in players.iterate do
			if other ~= player then
				P_PlayJingleMusic(other, musicName, nil, true, JT_OTHER)
			end
		end

		T.CustomMusic = 1
	end
	-- If no other survivor found, this player is last
	return true
end

T.UpdateTagHand = function(player)
	if not T.isInGamemode() then return end
	if not (player.mo and player.mo.valid) then return end
	if player.zzt_isIt then
		if not player.zzt_tagHand or not player.zzt_tagHand.valid then
			player.zzt_tagHand = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_TAGHAND)
		end

		local hand = player.zzt_tagHand
		if hand and hand.valid then
			local dist = 32*FRACUNIT
			local x = player.mo.x + P_ReturnThrustX(player.mo, player.mo.angle, dist)
			local y = player.mo.y + P_ReturnThrustY(player.mo, player.mo.angle, dist)
			local z = player.mo.z + 24*FRACUNIT

			P_SetOrigin(hand, x, y, z)
		end
	else
		if player.zzt_tagHand and player.zzt_tagHand.valid then
			P_RemoveMobj(player.zzt_tagHand)
			player.zzt_tagHand = nil
		end
	end
end


T.bombTimer = nil
T.lastBombSecond = nil

T.BombGameThink = function()
	if T.zzt_gamemode ~= "bomb" then return end
	if not T.isInGamemode() then return end

	-- Set the timer if not already set
	if not T.bombTimer then
		T.bombTimer = leveltime + (30 * TICRATE)
		T.lastBombSecond = nil
	end

	local ticksLeft = T.bombTimer - leveltime
	local secondsLeft = G_TicsToSeconds(ticksLeft)

	-- Last 5-second warning HUD messages
	if secondsLeft <= 5 and secondsLeft > 0 and T.lastBombSecond ~= secondsLeft then
		for player in players.iterate do
			if player.mo and player.valid and not player.spectator and not player.zzt_bombDead then
				T.ShowHUDMessage(player, "Bomb explodes in: " .. secondsLeft, TICRATE)
			end
		end
		T.lastBombSecond = secondsLeft
	end

	-- Not time yet
	if leveltime < T.bombTimer then return end

	-- Already waiting to transfer? Don't explode more people.
	if T.bombDelayTimer then
		if leveltime >= T.bombDelayTimer then
			-- Time to pick a new bomb carrier
			local candidates = {}
			for player in players.iterate do
				if player.mo and player.valid and not player.spectator and not player.zzt_bombDead then
					table.insert(candidates, player)
				end
			end

			if #candidates > 0 then
				local newIndex = P_RandomKey(#candidates) + 1
				local newPl = candidates[newIndex]
				newPl.zzt_isIt = true
				T.ShowHUDMessage(newPl, "\x85The bomb is now on you!", 2*TICRATE)
				T.bombTimer = leveltime + (30 * TICRATE)
				T.lastBombSecond = nil
			else
				-- No one left to pass to
				for player in players.iterate do
					if player.mo and player.valid and not player.spectator then
						T.ShowHUDMessage(player, "Everyone else exploded!", 3*TICRATE)
						T.CheckRoundEnd()
					end
				end
			end

			-- Clear delay timer
			T.bombDelayTimer = nil
		end

		return
	end

	-- Bomb explodes on the current "It" player
	local exploded = false
	for player in players.iterate do
		if player.mo and player.valid and not player.spectator and T.isIt(player) then
			P_KillMobj(player.mo, nil, nil)
			T.ShowHUDMessage(player, "You blew up!", 2*TICRATE)
			S_StartSound(nil, sfx_bomb, player)
			player.zzt_isIt = false
			player.zzt_bombDead = true
			exploded = true
			T.CheckRoundEnd()
		end
	end

	-- If someone exploded, start the delay timer
	if exploded then
		T.bombDelayTimer = leveltime + (5 * TICRATE) -- Wait 5 seconds
	end
end



T.AssignTaggersProperly = function()
	if not T.isInGamemode() then return end
	if T.zzt_gamemode == nil then return end

	-- Gather valid, non-spectating players
	local validPlayers = {}
	for player in players.iterate do
		if player.mo and player.valid and not player.spectator then
			table.insert(validPlayers, player)
		end
	end
	local count = #validPlayers
	if count == 1 then return end

	-- Clear current tag states
	for _, player in ipairs(validPlayers) do
		player.zzt_isIt = false
	end

	if T.zzt_gamemode == "juggernaut" then
		local chosenRunnerIndex = P_RandomKey(#validPlayers)
		local chosenRunner = validPlayers[chosenRunnerIndex]

		-- Mark all as taggers initially
		for _, player in ipairs(validPlayers) do
			player.zzt_isIt = true
		end

		-- Fallback if somehow no runner was selected
		if not chosenRunner then
			T.AssignTaggersProperly()
			return
		end

		-- Assign juggernaut
		chosenRunner.zzt_isIt = false
		chosenRunner.score = 100
		T.JuggernautHP = 100
		T.juggernautDead = false

		-- Optional huge mode
		if T.zzt_mod_hugejug == true then
			P_SetScale(chosenRunner.mo, FRACUNIT * 2)
			chosenRunner.zzt_isScaledUp = true -- custom flag
			chosenRunner.score = 250
			T.JuggernautHP = 250
		end

		return
	end

	-- Crown Mode: 50% runners, rest are It
	if T.zzt_gamemode == "crown" then
		local runnerCount = (count*2) / 5
		if runnerCount < 1 then runnerCount = 1 end
		if runnerCount >= count then runnerCount = count - 1 end

		local chosen = {}
		while runnerCount > 0 do
			local index = P_RandomKey(#validPlayers)
			local pick = validPlayers[index]
			if pick and not chosen[pick] then
				chosen[pick] = true
				runnerCount = runnerCount - 1
			end
		end

		-- Assign roles
		for _, player in ipairs(validPlayers) do
			if not chosen[player] then
				player.zzt_isIt = true
			end
		end
		return
	end

	-- Standard mode (not crown): assign taggers (50% of players)
	for _, player in ipairs(validPlayers) do
		player.zzt_isIt = false
	end

	local taggerCount = 1
	if count >= 4 then
		taggerCount = (count*2) / 5
		if T.zzt_gamemode == "zombie" then taggerCount = 1 end
		if taggerCount >= count then
			taggerCount = count - 1
		end
	end
	

	local chosen = {}
	while taggerCount > 0 do
		local index = P_RandomKey(#validPlayers)
		local pick = validPlayers[index]
		if pick and not chosen[pick] then
			pick.zzt_isIt = true
			chosen[pick] = true
			taggerCount = taggerCount - 1
		end
	end
end

T.CheckRoundEnd = function()
	if not T.isInGamemode() then return end

	local gm = T.zzt_gamemode or "zombie"
	local untaggedFound = false

	if gm == "tag" then return end

	local aliveCount = 0
	local deadCount = 0

	for player in players.iterate do
		if not player.valid or not player.mo then continue end
		if player.spectator then continue end

		if gm == "zombie" then
			if not T.isIt(player) then
				untaggedFound = true
				break
			end

		elseif gm == "freeze" then
			if not T.isIt(player) and not T.isFrozen(player) then
				untaggedFound = true
				break
			end

		elseif gm == "bomb" then
			if not player.zzt_bombDead then
				aliveCount = $ + 1
			else
				deadCount = $ + 1
			end
		elseif gm == "juggernaut" then
			if T.JuggernautHP > 0 then
				T.juggernautAlive = true
			else
				T.juggernautDead = true
			end
		end
	end

	-- Bomb-specific check: only one (or zero) alive players remain
	if gm == "bomb" and aliveCount <= 1 then
		print("Only one player left alive in Bomb mode. Ending round...")
		G_ExitLevel()
		T.zzt_gamemode = nil
		return
	end

	-- Juggernaut-specific check: if the runner is dead (score <= 0)
	if gm == "juggernaut" then
		if T.JuggernautHP <= 0 then
			print("Juggernaut runner eliminated! Ending round...")
			G_ExitLevel()
			T.zzt_gamemode = nil
			return
		end
	end

	-- Standard end check (zombie/freeze)
	if gm ~= "bomb" and gm ~= "juggernaut" and not untaggedFound then
		print("All players handled! Ending round...")
		G_ExitLevel()
		T.zzt_gamemode = nil
	end
end

T.TransferRunnerOnDeath = function(player)
	if not (player and player.valid and player.mo) then return end
	if player.spectator or player.zzt_isIt then return end
	if not T.isInGamemode() or T.zzt_gamemode == "zombie" or T.zzt_gamemode == "freeze" then return end
	if player.mo.health > 0 then return end

	local candidates = {}
	for other in players.iterate do
		if other ~= player
		and other.valid and other.mo and not other.spectator
		and other.zzt_isIt then
			table.insert(candidates, other)
		end
	end

	if #candidates == 0 then return end

	local newRunner = candidates[P_RandomRange(1, #candidates)]
	newRunner.zzt_isIt = false
	player.zzt_isIt = true

	T.ShowHUDMessage(newRunner, "\x82You are the new Runner!", 2*TICRATE)
	T.ShowHUDMessage(player, "\x85You are now It!", 2*TICRATE)
	S_StartSound(nil, sfx_s3k56, newRunner)
end

T.ShowHUDMessage = function(player, msg, time)
	if not (player and player.valid) then return end
	player.zzt_hudMsg = msg
	player.zzt_hudMsgTime = leveltime + (time or 2*TICRATE) -- default 2 seconds
end

addHook("PlayerThink", function(player)
	if not player.mo then return end
	if T.zzt_mod_bigmode then
		player.zzt_isScaledUp = true
	else
		player.zzt_isScaledUp = false
	end

	if T.zzt_mod_minimode then
		player.zzt_isScaledDown = true
	else
		player.zzt_isScaledDown = false
	end

	-- Maintain huge scale if flagged
	if player.zzt_isScaledUp then
		P_SetScale(player.mo, FRACUNIT * 2)
	elseif player.zzt_isScaledDown then
		P_SetScale(player.mo, FRACUNIT/2)
	else
		P_SetScale(player.mo, FRACUNIT)
	end

	--Increase coin every second
	if T.zzt_mod_bountiful and leveltime % 35 == 0 then
		player.rings = $ +1
	end
end)