local T = ZZT_ZombieTag



T.voteStartTime = nil  -- Time when voting started
  -- Track votes for each gamemode
T.isVotingActive = false  -- Flag to indicate if voting is active

-- Start Voting Phase (triggered at the start of a round)
T.StartVoting = function()
if not T.isInGamemode() then return end
    -- Reset votes and set voting time
	T.votes = {}
	for _, gametype in pairs(T.gametypes) do
		T.votes[gametype.name] = 0
	end

    T.voteStartTime = leveltime  -- Start the timer (leveltime is usually in ticks)
    T.isVotingActive = true
	T.zzt_gamemode = nil
end

function string.capitalize(str)
    return (str:gsub("^%l", string.upper))
end


-- Voting HUD: Display current vote counts
T.VoteHUD = function(v, player)
    if not T.isInGamemode() or not T.isVotingActive then return end

    local flags = V_PERPLAYER
    local x = -10
	local y = 10

    v.drawString(x, y, "Vote for Gamemode:", V_YELLOWMAP | flags, "left")
    y = y + 10

    for _, gamemode in ipairs(T.gametypes) do
        local name = gamemode.name
        local color = gamemode.color or V_ORANGEMAP  -- fallback color
        v.drawString(x, y, name:sub(1,1):upper() .. name:sub(2) .. ": " .. tostring(T.votes[name]), color | flags, "left")
        y = y + 10
    end
end

T.HandleVoting = function(player)
	if not T.isInGamemode() or not T.isVotingActive then return end
	if not (player.mo and player.mo.valid) then return end
    if leveltime - T.voteStartTime > 15 * TICRATE then
        -- End the voting after 15 seconds
        T.EndVoting()
        return
    end
	-- Initialize button history
	if player.vote_buttonhistory == nil then
		player.vote_buttonhistory = player.cmd.buttons
	end

	local now = player.cmd.buttons
	local before = player.vote_buttonhistory
	local newPress = now & ~before
	local weaponKey = newPress & BT_WEAPONMASK

	-- Only register vote if a NEW weapon key was pressed
	if weaponKey ~= 0 then
		local index = weaponKey
		local gametype = T.gametypes[index]
		if gametype then
			local gametypeName = gametype.name
			-- Use gametypeName to update votes
			local previousVote = player.zzt_vote
			if previousVote ~= gametypeName then
				if previousVote then
					T.votes[previousVote] = $ - 1
				end
				T.votes[gametypeName] = $ + 1
				player.zzt_vote = gametypeName
				player.zzt_abilitySwapDelay = 10
				S_StartSound(nil, sfx_menu1, player)
			end
		end
	end

	-- Store button state for next frame
	player.vote_buttonhistory = now
end

-- End Voting and decide which gamemode to select based on votes
T.EndVoting = function()
	if not T.isVotingActive then return end

	local highestVote = -1
	local winners = {}

	for _, gametype in ipairs(T.gametypes) do
		local name = gametype.name
		local count = T.votes[name] or 0
		if count > highestVote then
			highestVote = count
			winners = {name}
		elseif count == highestVote then
			table.insert(winners, name)
		end
	end

	local chosen = winners[P_RandomRange(1, #winners)]
	T.zzt_gamemode = chosen

	T.zzt_modifier = P_RandomRange(1, 20)

	local displayName = chosen:upper()

		if T.zzt_modifier == 14 then
		T.zzt_mod_bountiful = true
		displayName = "\x82[BOUNTIFUL] \x85" .. displayName
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 15 then
		T.zzt_mod_minimode = true
		displayName = "\x88[MINI] \x85" .. displayName
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 16 then
		T.zzt_mod_bigmode = true
		displayName = "\x87[HUGE] \x85" .. displayName
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 17 and T.zzt_gamemode == "juggernaut" then
		displayName = "\x89[MEGA] \x85" .. displayName
		T.zzt_mod_hugejug = true
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 18 and T.zzt_gamemode == "tag" then
		displayName = "\x86[DISGUISED] \x85" .. displayName
		T.zzt_mod_disguised = true
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 19 then
		displayName = "\x84[SPEEDY] \x85" .. displayName
		T.zzt_mod_speedy = true
		S_StartSound(nil, sfx_modi, player)
		elseif T.zzt_modifier == 20 then
		displayName = "\x83[BLITZ] \x85" .. displayName
		T.zzt_mod_blitz = true
		S_StartSound(nil, sfx_modi, player)
		end

	for player in players.iterate do
		if player and player.valid and not player.spectator then
			T.ShowHUDMessage(player, "\x83Gamemode selected: \x85" .. displayName, 5*TICRATE)
		end
	end

	T.isVotingActive = false
	T.AssignTaggersProperly()
end

T.ModifiedReset = function()
T.zzt_gamemode = nil
T.zzt_mod_speedy = false
T.zzt_mod_blitz = false
T.zzt_mod_disguised = false
T.zzt_mod_hugejug = false
T.zzt_mod_bigmode = false
T.zzt_mod_minimode = false
T.zzt_mod_bountiful = false
T.CustomMusic = 0
end


T.ModifierThink = function(player)
	if T.zzt_mod_speedy == true and T.isIt(player)
		player.powers[pw_sneakers] = 1
	end
end
